@@ -260,31 +260,31 @@ typedef struct {
  *****************************************************************************/
 
 /* server -> client */
+#define rfbFramebufferUpdate	0
+#define rfbSetColourMapEntries	1
+#define rfbBell			2
+#define rfbServerCutText	3
 
-#define rfbFramebufferUpdate 0
-#define rfbSetColourMapEntries 1
-#define rfbBell 2
-#define rfbServerCutText 3
-
-#define rfbClientAdv 15
-#define rfbClientExist 16
+#define rfbFileAdv		14
+#define rfbClientAdv		15
+#define rfbClientExist		16
 
 /* client -> server */
+#define rfbSetPixelFormat	0
+#define rfbFixColourMapEntries	1	/* not currently supported */
+#define rfbSetEncodings		2
+#define rfbFramebufferUpdateRequest	3
+#define rfbKeyEvent		4
+#define rfbPointerEvent		5
+#define rfbClientCutText	6
 
-#define rfbSetPixelFormat 0
-#define rfbFixColourMapEntries 1	/* not currently supported */
-#define rfbSetEncodings 2
-#define rfbFramebufferUpdateRequest 3
-#define rfbKeyEvent 4
-#define rfbPointerEvent 5
-#define rfbClientCutText 6
-
-#define rfbServerAdv 8
-#define rfbMultiFramebufferUpdate 9
-#define rfbAllowAdv 10
-#define rfbCastAdv 11
-#define rfbLockAdv 12
-#define rfbVncConnAdv 17
+#define rfbServerAdv		8
+#define rfbMultiFramebufferUpdate	9
+#define rfbAllowAdv		10
+#define rfbCastAdv		11
+#define rfbLockAdv		12
+#define rfbRecAdv		13
+#define rfbVncConnAdv		17
 
 
 /*****************************************************************************
@@ -293,14 +293,14 @@ typedef struct {
  *
  *****************************************************************************/
 
-#define rfbEncodingRaw 0
-#define rfbEncodingCopyRect 1
-#define rfbEncodingRRE 2
-#define rfbEncodingCoRRE 4
-#define rfbEncodingHextile 5
-#define rfbEncodingZlib 6
-#define rfbEncodingTight 7
-#define rfbEncodingZlibHex 8
+#define rfbEncodingRaw		0
+#define rfbEncodingCopyRect	1
+#define rfbEncodingRRE		2
+#define rfbEncodingCoRRE	4
+#define rfbEncodingHextile	5
+#define rfbEncodingZlib		6
+#define rfbEncodingTight	7
+#define rfbEncodingZlibHex	8
 
 /*
  * Special encoding numbers:
