@@ -11,7 +11,7 @@
 #include "interface.h"
 #include "support.h"
 
-char clientID[16];
+char clientID[IDLENGTH];
 GdkPixmap *pixmap_lock;
 extern GtkWidget *dialog_IDinput;
 extern GtkWidget *dialog_wait;
@@ -23,8 +23,9 @@ GtkWidget *lock_screen;
 extern GtkWidget *mini_window;
 GtkWidget *mini_drawingarea;
 GdkPixmap *mini_pixmap;
+extern int auto_flag;
 GtkWidget *dialog_exit;
-extern GdkFont *font;
+extern PangoFontDescription *font_dsp;
 
 
 /****************************************************************
@@ -95,6 +96,19 @@ on_dialog_IDinput_delete_event         (GtkWidget       *widget,
 }
 
 
+void
+on_dialog_IDinput_entry_editing_done   (GtkCellEditable *celleditable,
+                                        gpointer         user_data)
+{
+  if (strlen((char *)gtk_entry_get_text(GTK_ENTRY(celleditable))) < 1) {
+    return;
+  }
+  strcpy(clientID, (char *)gtk_entry_get_text(GTK_ENTRY(celleditable)));
+  printf("clientID(edit done): %s\n", clientID);
+
+}
+
+
 void
 on_dialog_IDinput_entry_changed       (GtkEditable     *editable,
                                         gpointer         user_data)
@@ -107,7 +121,11 @@ void
 on_dialog_IDinput_buttonOK_clicked     (GtkButton       *button,
                                         gpointer         user_data)
 {
-  strcpy(client_CB.client_ID, clientID);
+  if (strlen(clientID) < 1) {
+    return;
+  }
+  memcpy((char *)client_CB.client_ID, clientID, IDLENGTH);
+  printf("clientID(ok click): %s\n", client_CB.client_ID);
 
   gtk_grab_remove(dialog_IDinput);
   gtk_widget_destroy(dialog_IDinput);
@@ -118,7 +136,7 @@ on_dialog_IDinput_buttonOK_clicked     (GtkButton       *button,
   dialog_wait = create_dialog_wait();
   gtk_widget_show(dialog_wait);
   gtk_grab_add(dialog_wait);
-  draw_string_to_mini_window("Wait");
+  draw_string_to_mini_window(_("Wait"));
 }
 
 
@@ -157,7 +175,7 @@ on_dialog_wait_buttonStop_clicked      (GtkButton       *button,
   dialog_IDinput = create_dialog_IDinput();
   gtk_widget_show(dialog_IDinput);
   gtk_grab_add(dialog_IDinput);
-  draw_string_to_mini_window("Input ID");
+  draw_string_to_mini_window(_("Input ID"));
   gtk_main();
 }
 
@@ -170,7 +188,8 @@ void open_dialog_recvAdv(char *server_ID)
 {
   char recvAdv_msg[256];
 
-  dialog_recvAdv = (GtkWidget *)create_dialog_recvAdv();
+  dialog_recvAdv = create_dialog_recvAdv();
+  GtkWidget *recvAdv_label = lookup_widget(GTK_WIDGET(dialog_recvAdv), "recvAdv_label");
   //sprintf(recvAdv_msg, "\343\202\265\343\203\274\343\203\220\345\220\215\357\274\232%s\n\n\343\201\223\343\201\256\343\202\265\343\203\274\343\203\220\343\201\253\346\216\245\347\266\232\343\201\227\343\201\276\343\201\231\343\201\213?", serveradv.server_ID);
   sprintf(recvAdv_msg, _("Server Name : %s\n\nDo you connect to this server?"), server_ID);
   gtk_label_set_text(GTK_LABEL(recvAdv_label), recvAdv_msg);
@@ -188,6 +207,7 @@ on_dialog_recvAdv_delete_event         (GtkWidget       *widget,
   dialog_exit = create_dialog_exit();
   gtk_widget_show(dialog_exit);
   gtk_grab_add(dialog_exit);
+//  draw_string_to_mini_window(_("Quit"));
 
   return TRUE;
 }
@@ -201,7 +221,7 @@ on_dialog_recvAdv_buttonOK_clicked     (GtkButton       *button,
   gtk_grab_remove(dialog_recvAdv);
   gtk_widget_destroy(dialog_recvAdv);
   gtk_main_quit();
-  draw_string_to_mini_window("Connect");
+  draw_string_to_mini_window(_("Connect"));
 }
 
 
@@ -217,7 +237,7 @@ on_dialog_recvAdv_buttonCancel_clicked (GtkButton       *button,
   dialog_wait = create_dialog_wait();
   gtk_widget_show(dialog_wait);
   gtk_grab_add(dialog_wait);
-  draw_string_to_mini_window("Wait");
+  draw_string_to_mini_window(_("Wait"));
 }
 
 
@@ -271,7 +291,7 @@ create_lock_window (void)
                     G_CALLBACK (on_lock_screen_expose_event),
                     NULL);
 
-  draw_string_to_mini_window("Lock");
+  draw_string_to_mini_window(_("Lock"));
   gtk_widget_show(lock_window);
   gtk_grab_add(lock_window);
 }
@@ -283,8 +303,13 @@ on_lock_screen_configure_event    (GtkWidget       *widget,
                                         gpointer         user_data)
 {
   GdkGC *gc;
-  GdkFont *font;
   GdkColor color;
+  const char* fontsname = "Bitstream Vesa Sans 18";
+  font_dsp = pango_font_description_from_string(fontsname);
+  gtk_widget_modify_font (lock_window, font_dsp);
+
+  PangoLayout *layout = gtk_widget_create_pango_layout(lock_window, _("Now Locking ..."));
+  pango_layout_set_justify(layout, TRUE);
 
   if (pixmap_lock)
     gdk_pixmap_unref(pixmap_lock);
@@ -302,8 +327,7 @@ on_lock_screen_configure_event    (GtkWidget       *widget,
   gdk_draw_rectangle(pixmap_lock, gc, TRUE,
 		     0, 0, widget->allocation.width, widget->allocation.height);
 
-  font = gdk_fontset_load("-*-helvetica-bold-r-normal--*-300-*-*-*-*-ISO8859-1");
-  gdk_draw_string(pixmap_lock, font, widget->style->black_gc, 50, 100, "Now Locking ...");
+  gdk_draw_layout(pixmap_lock, widget->style->black_gc, 50, 100, layout);
 
   return TRUE;
 }
@@ -353,7 +377,7 @@ stop_lock(void)
 {
   if (GETLOCK_C) {
     SETLOCK_C(0);
-    draw_string_to_mini_window("Connect");
+    draw_string_to_mini_window(_("Connect"));
     //XUngrabKeyboard(dpy, CurrentTime);
     //XUngrabPointer(dpy, CurrentTime);
     gdk_keyboard_ungrab(GDK_CURRENT_TIME);
@@ -426,11 +450,11 @@ on_mini_drawingarea_configure_event    (GtkWidget       *widget,
 			       widget->allocation.width,
 			       widget->allocation.height, -1);
 
-  if (auto_flag != 0) {
-    draw_string_to_mini_window("Input ID");
-  } else {
-    draw_string_to_mini_window("Connect");
-  }
+//  if (auto_flag != 0) {
+    draw_string_to_mini_window(_("Input ID"));
+//  } else {
+//    draw_string_to_mini_window(_("Connect"));
+//  }
 
   return TRUE;
 }
@@ -461,12 +485,19 @@ draw_string_to_mini_window(char *string)
   GdkRectangle gdkrect;
   GdkGC *gc;
   GdkColor color;
+  const char* fontsname = "Bitstream Vesa Sans 24";
+//  const char* fontsname = "IPAUIGothic 24";
+  font_dsp = pango_font_description_from_string(fontsname);
+  gtk_widget_modify_font (mini_window, font_dsp);
+
+  PangoLayout *layout = gtk_widget_create_pango_layout(mini_window, _(string));
+  pango_layout_set_justify(layout, TRUE);
 
   if (notice_flag == 1) {
     gc = mini_drawingarea->style->white_gc;
     gdk_draw_rectangle(mini_pixmap, gc, TRUE,
 		       0, 0, mini_drawingarea->allocation.width, mini_drawingarea->allocation.height);
-    gdk_draw_string(mini_pixmap, font, mini_drawingarea->style->black_gc, 1, 20, string);
+    gdk_draw_layout(mini_pixmap, mini_drawingarea->style->black_gc, 0, 0, layout);
 
     gdkrect.x = 0;
     gdkrect.y = 0;
@@ -476,6 +507,17 @@ draw_string_to_mini_window(char *string)
   }
 }
 
+// 1.4.2 non implementation
+void change_status_mini_window() {
+    char status[STRLENGTH];
+    switch (client_CB.state) {
+        case 0:
+            strcpy(status, _("Wait"));
+            break;
+//        default:
+    }
+    draw_string_to_mini_window(status);
+}
 
 gboolean
 on_dialog_exit_delete_event            (GtkWidget       *widget,
@@ -496,7 +538,7 @@ on_dialog_exit_buttonOK_clicked        (GtkButton       *button,
   gtk_widget_destroy(dialog_exit);
   gtk_grab_remove(dialog_exit);
   gtk_widget_destroy(mini_window);
-//  CloseVNCServer();
+  CloseVNCServer();
   exit(0);
 }
 
@@ -509,4 +551,3 @@ on_dialog_exit_buttonCancel_clicked    (GtkButton       *button,
   gtk_grab_remove(dialog_exit);
 }
 
-
