@@ -131,3 +131,29 @@ on_drawing_window_delete_event         (GtkWidget       *widget,
                                         GdkEvent        *event,
                                         gpointer         user_data);
 
+
+void
+on_dialog_IDinput_entry_editing_done   (GtkCellEditable *celleditable,
+                                        gpointer         user_data);
+
+gboolean
+on_entry_focus_in_event                (GtkWidget       *widget,
+                                        GdkEventFocus   *event,
+                                        gpointer         user_data);
+
+gboolean
+on_entry_focus_out_event               (GtkWidget       *widget,
+                                        GdkEventFocus   *event,
+                                        gpointer         user_data);
+
+void
+on_entry_realize                       (GtkWidget       *widget,
+                                        gpointer         user_data);
+
+void
+on_entry_unrealize                     (GtkWidget       *widget,
+                                        gpointer         user_data);
+
+void
+on_dialog_IDinput_entry_changed        (GtkEditable     *editable,
+                                        gpointer         user_data);
