@@ -6,11 +6,14 @@
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 #include <X11/Xlib.h>
+#include <errno.h>
+#include <pthread.h>
+
+
 #include "callbacks.h"
 #include "interface.h"
 #include "support.h"
 #include "vncviewer.h"
-#include <errno.h>
 
 void makeTable_dialog_viewset(int pCnt);
 void makeList_dialog_viewset(int pCnt);
@@ -63,17 +66,27 @@ GtkWidget *dialog_send_comboboxentry2;
 char fullscreen_flag;
 
 extern GtkWidget *dialog_IDinput;
-char server_name[16];
+char server_name[IDLENGTH];
 
 GtkWidget *dialog_lock;
 GtkWidget *dialog_lock_table;
 
+#if 0
+GtkWidget *dialog_record;
+GtkWidget *dialog_record_table;
+int recording (int rec_time, int fps, char *geometry);
+#endif
+
 GtkWidget *dialog_quit;
 
 GtkWidget *dialog_version;
 
 GtkWidget *dialog_notice;
 
+extern void *AcceptLoop (void *arg);
+extern int SendRecvLoop (int acc, int fsize, char *fname);
+
+
 /*******************************************************************
  *ᥤ󥦥ɥФ륤٥ȤΥХåؿ
  *******************************************************************/
@@ -112,7 +125,7 @@ on_notebook_switch_page                (GtkNotebook     *notebook,
   for (dCnt = 0; dCnt < TABGROUP_MAX; dCnt++) {
     if ((c_num = draw_info[current_page].client_num[dCnt]) == EMPTY) continue;
     if (server_CB.client_grp[c_num] == NULL) continue;
-    SendAllowMsg(c_num, server_CB.client_grp[c_num]->client_ID);
+    SendAllowMsg(c_num, (char *)server_CB.client_grp[c_num]->client_ID);
   }
 }
 
@@ -305,8 +318,8 @@ on_send_activate                       (GtkMenuItem     *menuitem,
     for (dCnt = 0; dCnt < TABGROUP_MAX; dCnt++) {
       if (draw_info[pCnt].client_num[dCnt] == EMPTY) continue;
       if (GETRECV(draw_info[pCnt].client_num[dCnt]) || \
-	  (strcmp(server_CB.client_grp[draw_info[pCnt].client_num[dCnt]]->client_ID, \
-	  server_CB.server_ID) == 0))
+	  (strcmp((char *)server_CB.client_grp[draw_info[pCnt].client_num[dCnt]]->client_ID, \
+	  (char *)server_CB.server_ID) == 0))
 	gtk_toggle_button_set_inconsistent (GTK_TOGGLE_BUTTON (dialog_send_checkbutton2[pCnt][dCnt]), TRUE);
       else
 	g_signal_connect ((gpointer) dialog_send_checkbutton2[pCnt][dCnt], "toggled",
@@ -343,8 +356,8 @@ on_lock_activate                       (GtkMenuItem     *menuitem,
   for (pCnt = 0; pCnt < tab_num; pCnt++) {
     for (dCnt = 0; dCnt < TABGROUP_MAX; dCnt++) {
       if (draw_info[pCnt].client_num[dCnt] == EMPTY) continue;
-      if (strcmp(server_CB.client_grp[draw_info[pCnt].client_num[dCnt]]->client_ID, \
-	  server_CB.server_ID) == 0) {
+      if (strcmp((char *)server_CB.client_grp[draw_info[pCnt].client_num[dCnt]]->client_ID, \
+	  (char *)server_CB.server_ID) == 0) {
 	gtk_toggle_button_set_inconsistent (GTK_TOGGLE_BUTTON (dialog_send_checkbutton2[pCnt][dCnt]), TRUE);
       } else {
         gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dialog_send_checkbutton2[pCnt][dCnt]), \
@@ -373,6 +386,7 @@ on_version_activate                    (GtkMenuItem     *menuitem,
 {
   dialog_version = create_dialog_version();
   gtk_widget_show(dialog_version);
+  gtk_grab_add(dialog_version);
 }
 
 
@@ -401,7 +415,10 @@ void
 on_dialog_IDinput_buttonOK_clicked     (GtkButton       *button,
                                         gpointer         user_data)
 {
-  strcpy(server_CB.server_ID, server_name);
+  //printf("ID Length: %d\n", strlen(server_name));
+  if (strlen(server_name) < 1) return -1;
+//  strcpy((char *)server_CB.server_ID, server_name);
+  memcpy((char *)server_CB.server_ID, server_name, IDLENGTH);
   gtk_grab_remove(dialog_IDinput);
   gtk_widget_destroy(dialog_IDinput);
   return gtk_main_quit();
@@ -471,6 +488,7 @@ void makeTable_dialog_viewset(int pCnt)
   gtk_widget_show (dialog_viewset_entry[pCnt]);
   gtk_box_pack_start (GTK_BOX (vbox[pCnt]), dialog_viewset_entry[pCnt], FALSE, FALSE, 0);
   gtk_entry_set_text (GTK_ENTRY (dialog_viewset_entry[pCnt]), _(draw_info[pCnt].group_name));
+  gtk_entry_set_max_length (GTK_ENTRY (dialog_viewset_entry[pCnt]), 8);
 
   hbox_row[pCnt] = gtk_hbox_new (FALSE, 0);
   gtk_widget_show (hbox_row[pCnt]);
@@ -537,7 +555,7 @@ void makeList_dialog_viewset(int pCnt)
   dialog_viewset_list[pCnt] = gtk_list_new ();
   for (dCnt = 0; dCnt < TABGROUP_MAX; dCnt++) {
     if (dialog_viewset_clientnum[pCnt][dCnt] == EMPTY) continue;
-    item = gtk_list_item_new_with_label(server_CB.client_grp[dialog_viewset_clientnum[pCnt][dCnt]]->client_ID);
+    item = gtk_list_item_new_with_label((gchar *)server_CB.client_grp[dialog_viewset_clientnum[pCnt][dCnt]]->client_ID);
     gtk_container_add(GTK_CONTAINER(dialog_viewset_list[pCnt]), item);
     gtk_widget_show(item);
   }
@@ -556,7 +574,6 @@ on_dialog_viewset_delete_event                (GtkWidget       *widget,
 					       GdkEvent        *event,
 					       gpointer         user_data)
 {
-  GtkWidget *dialog_viewset = lookup_widget(GTK_WIDGET(dialog_viewset), "dialog_viewset");
   gtk_grab_remove(dialog_viewset);
   gtk_widget_destroy(dialog_viewset);
   dialog_viewset = NULL;
@@ -594,7 +611,7 @@ on_dialog_viewset_buttonOK_clicked             (GtkButton       *button,
       server_CB.client_grp[num]->draw_area = dCnt;
       if (((dCnt < newareamax) && (pCnt == current_page)) || (GETVIEW(num) || (GETSEND(num)))) {
 	/* Connect VNC */
-	SendAllowMsg(num, server_CB.client_grp[num]->client_ID);
+	SendAllowMsg(num, (char *)server_CB.client_grp[num]->client_ID);
       } else {
 	/* Cut VNCconnection */
 	CutVNC(num);
@@ -760,7 +777,7 @@ add_to_dialog_viewset(int cnum)
       if (dialog_viewset_clientnum[pCnt][dCnt] == cnum) return;
       if (dialog_viewset_clientnum[pCnt][dCnt] == EMPTY) {
 	dialog_viewset_clientnum[pCnt][dCnt] = cnum;
-	item = gtk_list_item_new_with_label(server_CB.client_grp[cnum]->client_ID);
+	item = gtk_list_item_new_with_label((char *)server_CB.client_grp[cnum]->client_ID);
 	gtk_container_add(GTK_CONTAINER(dialog_viewset_list[pCnt]), item);
 	gtk_widget_show(item);
 	flag = 1;
@@ -791,12 +808,12 @@ expanding_screen()
     GtkWidget *aspectframe = lookup_widget(GTK_WIDGET(window_ex), "aspectframe");
     drawingarea_ex = lookup_widget(GTK_WIDGET(window_ex), "drawingarea_ex");
 
-    gtk_label_set_text (GTK_LABEL(label_ex), server_CB.client_grp[clientNum]->client_ID);
+    gtk_label_set_text (GTK_LABEL(label_ex), (gchar *)server_CB.client_grp[clientNum]->client_ID);
     gtk_widget_show(label_ex);
 
     float aspect = (float)server_CB.client_grp[clientNum_ex]->serverInitMsg.framebufferWidth / (float)server_CB.client_grp[clientNum_ex]->serverInitMsg.framebufferHeight;
     gtk_aspect_frame_set(GTK_ASPECT_FRAME(aspectframe), 0.5, 0.5, aspect, FALSE);
-    gtk_widget_set_size_request (aspectframe, exwindowWidth, exwindowHeight);
+    gtk_window_set_default_size(GTK_WINDOW(window_ex), exwindowWidth, exwindowHeight);
     gtk_widget_show(aspectframe);
 
     gtk_widget_set_events (drawingarea_ex, GDK_EXPOSURE_MASK | GDK_POINTER_MOTION_MASK
@@ -822,7 +839,7 @@ expanding_screen()
     smallimage_ex = gdk_image_new (GDK_IMAGE_FASTEST, gdkvis,
 				   server_CB.client_grp[clientNum_ex]->serverInitMsg.framebufferWidth,
 				   server_CB.client_grp[clientNum_ex]->serverInitMsg.framebufferHeight);
-    gtk_label_set_text(GTK_LABEL(label_ex), server_CB.client_grp[clientNum_ex]->client_ID);
+    gtk_label_set_text(GTK_LABEL(label_ex), (gchar *)server_CB.client_grp[clientNum_ex]->client_ID);
     clientNum = clientNum_ex;
     draw_image(pixmap_ex, gdkGC, gdkimage[clientNum_ex], 0, 0, 0, 0,
                gdkimage[clientNum_ex]->width, gdkimage[clientNum_ex]->height);
@@ -1083,6 +1100,7 @@ create_rightclickmenu1 (gpointer user_data)
   GtkWidget *screen_lock;
   GtkWidget *screen_send;
   GtkWidget *screen_send_stop;
+//  GtkWidget *screen_record;
 
   rightclickmenu1 = gtk_menu_new ();
 
@@ -1110,6 +1128,10 @@ create_rightclickmenu1 (gpointer user_data)
   gtk_widget_show (screen_send_stop);
   gtk_container_add (GTK_CONTAINER (rightclickmenu1), screen_send_stop);
 
+//  screen_record = gtk_menu_item_new_with_mnemonic (_("Record screen"));
+//  gtk_widget_show (screen_record);
+//  gtk_container_add (GTK_CONTAINER (rightclickmenu1), screen_record);
+
   g_signal_connect ((gpointer) viewset, "activate",
                     G_CALLBACK (on_viewset_activate),
                     NULL);
@@ -1128,6 +1150,9 @@ create_rightclickmenu1 (gpointer user_data)
   g_signal_connect ((gpointer) screen_send_stop, "activate",
                     G_CALLBACK (on_screen_send_stop_activate),
                     (gpointer)c_num);
+//  g_signal_connect ((gpointer) screen_record, "activate",
+//                    G_CALLBACK (on_screen_record_activate),
+//                    (gpointer)c_num);
 
   return rightclickmenu1;
 }
@@ -1216,7 +1241,7 @@ on_screen_lock_activate                (GtkMenuItem     *menuitem,
 {
   int c_num = (int)user_data;
 
-  if (strcmp(server_CB.client_grp[c_num]->client_ID, server_CB.server_ID) == 0) return;
+  if (strcmp((char *)server_CB.client_grp[c_num]->client_ID, (char *)server_CB.server_ID) == 0) return;
   SETLOCK(c_num, !GETLOCK(c_num));
   SendLockMsg(GETLOCK(c_num), c_num);
 }
@@ -1236,10 +1261,11 @@ on_screen_send_activate                (GtkMenuItem     *menuitem,
     on_send_activate(menuitem, NULL);
     gtk_entry_set_text (GTK_ENTRY (GTK_BIN(dialog_send_comboboxentry1)->child), _(draw_info[p].group_name));
     gtk_entry_set_text (GTK_ENTRY (GTK_BIN(dialog_send_comboboxentry2)->child),
-		        _(server_CB.client_grp[draw_info[p].client_num[d]]->client_ID));
+		        _((char *)server_CB.client_grp[draw_info[p].client_num[d]]->client_ID));
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dialog_send_checkbutton2[p][d]), FALSE);
     gtk_toggle_button_set_inconsistent (GTK_TOGGLE_BUTTON (dialog_send_checkbutton2[p][d]), TRUE);
-    src_checkbutton = dialog_send_checkbutton2[p][d];
+    if (strcmp((char *)server_CB.client_grp[draw_info[p].client_num[d]]->client_ID, (char *)server_CB.server_ID) != 0)
+      src_checkbutton = dialog_send_checkbutton2[p][d];
   }
 }
 
@@ -1256,6 +1282,151 @@ on_screen_send_stop_activate           (GtkMenuItem     *menuitem,
 }
 
 
+#if 0
+/*******************************************************************
+ *ϿǻѤؿ
+ *******************************************************************/
+void
+on_screen_record_activate              (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+  int pCnt, dCnt, flag = 0;
+  dialog_record = create_dialog_record();
+  dialog_record_table = makeTable_userList();
+  GtkWidget *checkbutton0 = lookup_widget(dialog_record, "dialog_record_checkbutton0");
+
+  // can't setting resolution (widget hide)
+  GtkWidget *hbox5 = lookup_widget(dialog_record, "hbox5");
+  gtk_widget_hide(hbox5);
+
+  for (pCnt = 0; pCnt < tab_num; pCnt++) {
+    for (dCnt = 0; dCnt < TABGROUP_MAX; dCnt++) {
+      if (draw_info[pCnt].client_num[dCnt] == EMPTY) continue;
+      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dialog_send_checkbutton2[pCnt][dCnt]), \
+		      GETREC(draw_info[pCnt].client_num[dCnt])); g_signal_connect ((gpointer) dialog_send_checkbutton2[pCnt][dCnt], "toggled",
+		      G_CALLBACK (on_dialog_lock_checkbutton2_toggled), 
+		      (gpointer) ((pCnt << 8)+dCnt));
+      flag += gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON ( dialog_send_checkbutton2[pCnt][dCnt]));
+    }
+    g_signal_connect ((gpointer) dialog_send_checkbutton1[pCnt], "toggled", 
+		    G_CALLBACK (on_dialog_lock_checkbutton1_toggled),
+		    (gpointer) pCnt);
+    g_signal_connect ((gpointer) checkbutton0, "toggled", 
+		    G_CALLBACK (on_dialog_lock_checkbutton0_toggled),
+		    (gpointer) pCnt);
+  }
+
+  // start or stop record button hide
+  if (flag) {
+    GtkWidget *button_record_start = lookup_widget(dialog_record, "button_record_start");
+    gtk_widget_hide(button_record_start);
+  }
+  else {
+    GtkWidget *button_record_stop = lookup_widget(dialog_record, "button_record_stop");
+    gtk_widget_hide(button_record_stop);
+  }
+  GtkWidget *dialog_record_viewport = lookup_widget(GTK_WIDGET(dialog_record), "dialog_record_viewport");
+  gtk_container_add (GTK_CONTAINER (dialog_record_viewport), dialog_record_table);
+  gtk_widget_show(dialog_record_table);
+  gtk_widget_show(dialog_record);
+  gtk_grab_add(dialog_record);
+}
+
+
+void
+on_dialog_rec_start_button_clicked     (GtkButton       *button,
+                                        gpointer         user_data)
+{
+  // start <-> stop change
+  GtkWidget *button_record_start = lookup_widget(dialog_record, "button_record_start");
+  GtkWidget *button_record_stop = lookup_widget(dialog_record, "button_record_stop");
+  gtk_widget_hide(button_record_start);
+  gtk_widget_show(button_record_stop);
+
+  // get recording time, FPS, and resolution
+  GtkWidget *sppinbutton_record = lookup_widget(GTK_WIDGET(dialog_record), "spinbutton_rec_time");
+  GtkWidget *spinbutton_fps = lookup_widget(GTK_WIDGET(dialog_record), "spinbutton_fps");
+  GtkWidget *combo_entry = lookup_widget(GTK_WIDGET(dialog_record), "combo_entry1");
+
+  int rec_time = gtk_spin_button_get_value(GTK_SPIN_BUTTON(sppinbutton_record));
+  int fps = gtk_spin_button_get_value(GTK_SPIN_BUTTON(spinbutton_fps));
+  gchar geometry[IDLENGTH];
+  strcpy(geometry, gtk_entry_get_text (GTK_ENTRY(combo_entry))); 
+
+  printf("recording time:%d, fps:%d, geometry(resolution):%s\n", rec_time, fps, geometry);
+
+  recording(rec_time, fps, geometry);
+}
+
+
+void
+on_dialog_rec_stop_button_clicked      (GtkButton       *button,
+                                        gpointer         user_data)
+{
+  // start <-> stop change
+  GtkWidget *button_record_start = lookup_widget(dialog_record, "button_record_start");
+  GtkWidget *button_record_stop = lookup_widget(dialog_record, "button_record_stop");
+  gtk_widget_show(button_record_start);
+  gtk_widget_hide(button_record_stop);
+
+  // all chech button clear
+  GtkWidget *checkbutton0 = lookup_widget(dialog_record, "dialog_record_checkbutton0");
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbutton0), TRUE);
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbutton0), FALSE);
+
+  recording(-1, -1, "");
+}
+
+
+void
+on_dialog_rec_close_button_clicked     (GtkButton       *button,
+                                        gpointer         user_data)
+{
+  gtk_grab_remove(dialog_record);
+  gtk_widget_destroy(dialog_record);
+}
+
+
+gboolean
+on_dialog_record_delete_event          (GtkWidget       *widget,
+                                        GdkEvent        *event,
+                                        gpointer         user_data)
+{
+  gtk_grab_remove(dialog_record);
+  gtk_widget_destroy(dialog_record);
+  return TRUE;
+}
+
+
+int
+recording (int rec_time, int fps, char *geometry)
+{
+  int p, d, num, rec_flag;
+
+  for (p = 0; p < tab_num; p++) {
+    for (d = 0; d < TABGROUP_MAX; d++) {
+      if ((num= draw_info[p].client_num[d]) == EMPTY) continue;
+
+      rec_flag = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dialog_send_checkbutton2[p][d]));
+
+      printf("#####[rec control:client:%d(%s) flag %d]#####\n", num, inet_ntoa(server_CB.client_grp[num]->client_ip), rec_flag);
+      if (rec_time > 0 && (GETREC(num) || rec_flag)) {
+        // record start
+        SendRecMsg(START, server_CB.client_grp[num]->ctldsp, rec_time, fps, geometry);
+        SETREC(num, START);
+      } else if (rec_time < 0) {
+        // record stop
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dialog_send_checkbutton2[p][d]), FALSE);
+        SendRecMsg(STOP, server_CB.client_grp[num]->ctldsp, 0, 0, NULL);
+	SETREC(num, STOP);
+      }
+    }
+  }
+  return 0;
+}
+
+#endif
+
 /*******************************************************************
  *ǻѤؿ
  *******************************************************************/
@@ -1305,7 +1476,7 @@ void makeCombobox2_dialog_send(void)
 GtkWidget *makeTable_userList(void)
 {
   int pCnt, dCnt, c_num;
-  char tab[16];
+  char tab[16]; // TAB_MAX ?
   GtkWidget *table;
   GtkWidget *label;
   GtkWidget *vbox[TAB_MAX];
@@ -1348,7 +1519,7 @@ GtkWidget *makeTable_userList(void)
     /* Ŀcheck button κ */
     for (dCnt = 0; dCnt < TABGROUP_MAX; dCnt++) {
       if ((c_num = draw_info[pCnt].client_num[dCnt]) == EMPTY) continue;
-      dialog_send_checkbutton2[pCnt][dCnt] = gtk_check_button_new_with_mnemonic (_(server_CB.client_grp[c_num]->client_ID));
+      dialog_send_checkbutton2[pCnt][dCnt] = gtk_check_button_new_with_mnemonic (_((char *)server_CB.client_grp[c_num]->client_ID));
       gtk_widget_show (dialog_send_checkbutton2[pCnt][dCnt]);
       gtk_box_pack_start(GTK_BOX(vbox[pCnt]), dialog_send_checkbutton2[pCnt][dCnt], TRUE, TRUE, 0);
     }
@@ -1388,7 +1559,7 @@ GtkWidget *makeTable_send_state(void)
       }
 
       /* ֤ꤹ뤿check buttonκ */
-      dialog_send_checkbutton3[castnum] = gtk_check_button_new_with_mnemonic (_(server_CB.client_grp[src_num]->client_ID));
+      dialog_send_checkbutton3[castnum] = gtk_check_button_new_with_mnemonic (_((char *)server_CB.client_grp[src_num]->client_ID));
       gtk_widget_show (dialog_send_checkbutton3[castnum]);
       gtk_table_attach (GTK_TABLE (table), dialog_send_checkbutton3[castnum], 0, 1, 2*mcCnt, 2*mcCnt+1,
 		        (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
@@ -1412,7 +1583,7 @@ GtkWidget *makeTable_send_state(void)
   for (dstCnt = 0; dstCnt < MAXCLIENT; dstCnt++) {
     if (server_CB.client_grp[dstCnt] == NULL) continue;
     if ((server_CB.client_grp[dstCnt]->recv_grp != EMPTY)) {
-      dialog_send_checkbutton4[dstCnt] = gtk_check_button_new_with_mnemonic (_(server_CB.client_grp[dstCnt]->client_ID));
+      dialog_send_checkbutton4[dstCnt] = gtk_check_button_new_with_mnemonic (_((char *)server_CB.client_grp[dstCnt]->client_ID));
       gtk_widget_show (dialog_send_checkbutton4[dstCnt]);
       gtk_box_pack_start(GTK_BOX(vbox[server_CB.client_grp[dstCnt]->recv_grp]), dialog_send_checkbutton4[dstCnt], TRUE, TRUE, 0);
       gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dialog_send_checkbutton4[dstCnt]), GETRECV(dstCnt));
@@ -1559,7 +1730,7 @@ on_dialog_send_comboboxentry1_changed      (GtkComboBox     *combobox,
 	if (num == EMPTY) continue;
 	if (!GETSEND(num)) {
 	  gtk_combo_box_append_text (GTK_COMBO_BOX (dialog_send_comboboxentry2),
-				     _(server_CB.client_grp[num]->client_ID));
+				     _((char *)server_CB.client_grp[num]->client_ID));
 	}
       }
       return;
@@ -1586,7 +1757,7 @@ on_dialog_send_comboboxentry2_changed      (GtkComboBox     *combobox,
     for (p = 0; p < tab_num; p++) {
       for (d = 0; d < TABGROUP_MAX; d++) {
 	if (draw_info[p].client_num[d] == EMPTY) continue;
-        if ((strcmp(server_CB.client_grp[draw_info[p].client_num[d]]->client_ID, client_name) == 0) && (strcmp(server_CB.server_ID, client_name) != 0)) {
+        if ((strcmp((char *)server_CB.client_grp[draw_info[p].client_num[d]]->client_ID, client_name) == 0) && (strcmp((char *)server_CB.server_ID, client_name) != 0)) {
 	  src_checkbutton = dialog_send_checkbutton2[p][d];
 	  gtk_toggle_button_set_inconsistent (GTK_TOGGLE_BUTTON (src_checkbutton), TRUE);
 	  return;
@@ -1624,7 +1795,7 @@ send_start(void)
   if (client_name != NULL) {
     for (src_client = 0; src_client < MAXCLIENT; src_client++) {
       if (server_CB.client_grp[src_client] == NULL) continue;
-      if (strcmp(server_CB.client_grp[src_client]->client_ID, client_name) == 0) {
+      if (strcmp((char *)server_CB.client_grp[src_client]->client_ID, client_name) == 0) {
         send_flag = TRUE;
 	break;
       }
@@ -1643,7 +1814,7 @@ send_start(void)
     for (p = 0; p < tab_num; p++) {
       for (d = 0; d < TABGROUP_MAX; d++) {
 	if ((c_num = draw_info[p].client_num[d]) == EMPTY) continue;
-	if (strcmp(server_CB.client_grp[c_num]->client_ID, server_CB.server_ID) == 0) continue;
+	if (strcmp((char *)server_CB.client_grp[c_num]->client_ID, (char *)server_CB.server_ID) == 0) continue;
 	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dialog_send_checkbutton2[p][d]))) {
 	  if ((m = server_CB.client_grp[src_client]->cast_grp) != EMPTY) {
 	    server_CB.cast_src[m] = EMPTY;
@@ -1710,12 +1881,12 @@ send_start(void)
 	  exit (0);
 	}
 	server_CB.client_grp[c_num]->calcrate_tag = \
-	  gtk_timeout_add (CALC_INTERVAL, calcRateCallback, (gpointer)c_num);
+	  gtk_timeout_add (CALC_INTERVAL, (GtkFunction)calcRateCallback, (gpointer)c_num);
 	if (Resend_flag)
 	  server_CB.client_grp[c_num]->resend_tag = \
-	    gtk_timeout_add (RESEND_INTERVAL, resendCallback, (gpointer)c_num);
+	    gtk_timeout_add (RESEND_INTERVAL, (GtkFunction)resendCallback, (gpointer)c_num);
 	calcRateCallback ((gpointer)c_num);
-	SendAllowMsg(c_num, server_CB.client_grp[c_num]->client_ID);
+	SendAllowMsg(c_num, (char *)server_CB.client_grp[c_num]->client_ID);
       } else {
 	(*shm_asym[c_num]).flag = CAST_STOP;
 	int shmctlck;
@@ -1807,7 +1978,7 @@ on_dialog_send_buttonApply_clicked         (GtkButton       *button,
   for (pCnt = 0; pCnt < tab_num; pCnt++) {
     for (dCnt = 0; dCnt < TABGROUP_MAX; dCnt++) {
       if (draw_info[pCnt].client_num[dCnt] == EMPTY) continue;
-      if (GETRECV(draw_info[pCnt].client_num[dCnt]) || (strcmp(server_CB.client_grp[draw_info[pCnt].client_num[dCnt]]->client_ID, server_CB.server_ID) == 0)) {
+      if (GETRECV(draw_info[pCnt].client_num[dCnt]) || (strcmp((char *)server_CB.client_grp[draw_info[pCnt].client_num[dCnt]]->client_ID, (char *)server_CB.server_ID) == 0)) {
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dialog_send_checkbutton2[pCnt][dCnt]), TRUE);
 	gtk_toggle_button_set_inconsistent (GTK_TOGGLE_BUTTON (dialog_send_checkbutton2[pCnt][dCnt]), TRUE);
       } else
@@ -1934,7 +2105,7 @@ void locking_start(void)
   for (p = 0; p < tab_num; p++) {
     for (d = 0; d < TABGROUP_MAX; d++) {
       if ((num= draw_info[p].client_num[d]) == EMPTY) continue;
-      if (strcmp(server_CB.client_grp[num]->client_ID, server_CB.server_ID) == 0) continue;
+      if (strcmp((char *)server_CB.client_grp[num]->client_ID, (char *)server_CB.server_ID) == 0) continue;
       lock_flag = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dialog_send_checkbutton2[p][d]));
       if ((lock_flag && !GETLOCK(num)) || (!lock_flag && GETLOCK(num))) {
 	SETLOCK(num, lock_flag);
@@ -1992,6 +2163,7 @@ on_dialog_version_delete_event         (GtkWidget       *widget,
                                         gpointer         user_data)
 {
   gtk_widget_destroy(dialog_version);
+  gtk_grab_remove(dialog_version);
 
   return TRUE;
 }
@@ -2002,6 +2174,7 @@ on_dialog_version_buttonOK_clicked     (GtkButton       *button,
                                         gpointer         user_data)
 {
   gtk_widget_destroy(dialog_version);
+  gtk_grab_remove(dialog_version);
 }
 
 
@@ -2033,6 +2206,7 @@ on_dialog_notice_delete_event          (GtkWidget       *widget,
                                         gpointer         user_data)
 {
   gtk_widget_destroy(dialog_notice);
+//  gtk_main_quit();
   return TRUE;
 }
 
@@ -2042,6 +2216,7 @@ on_dialog_notify_okbutton_clicked      (GtkButton       *button,
                                         gpointer         user_data)
 {
   gtk_widget_destroy(dialog_notice);
+//  gtk_main_quit();
 }
 
 
