@@ -12,4 +12,6 @@ rm -f /usr/share/locale/zh_CN/LC_MESSAGES/multivnc_server.mo
 rm -f /usr/share/locale/zh_CN/LC_MESSAGES/multivnc_client.mo
 rm -f /usr/share/locale/vi/LC_MESSAGES/multivnc_server.mo
 rm -f /usr/share/locale/vi/LC_MESSAGES/multivnc_client.mo
+rm -f /usr/share/locale/ro/LC_MESSAGES/multivnc_server.mo
+rm -f /usr/share/locale/ro/LC_MESSAGES/multivnc_client.mo
 echo "MultiVNC Uninstall COMPLETE!"
