@@ -1,17 +1,16 @@
-#include <stdio.h>
+#include "vncviewer.h"
 #include <sys/socket.h>
 #include <resolv.h>
 #include <arpa/inet.h>
-#include <errno.h>
 #include <netinet/in.h>
-#include "vncviewer.h"
+#include <errno.h>
 #include <mysql.h>
 
-#include <sys/shm.h>
 #include <unistd.h>
+#include <sys/shm.h>
 #include <sys/ipc.h>
 #include <sys/sem.h>
-
+#include <signal.h>
 
 typedef struct _GdkIOClosure GdkIOClosure;
 struct _GdkIOClosure
@@ -27,6 +26,7 @@ static void vncSocketCallback(gpointer clientData, gint fd, GdkInputCondition co
 static Bool CheckDrawingarea(int);
 static Bool DiscoverDrawingarea(int);
 void CheckLockClient(void);
+extern char *swf_file;
 
 int getClientFromCtldsp (int fd);
 int getClientFromVncdsp (int fd);
@@ -34,10 +34,14 @@ void SendCastMsg (char flag, int clientNum, char fullscreen);
 int ConnectMyself ();
 Bool AuthDB (char *client_ID);
 
+
 extern int multi_ttl;
 extern GtkWidget *main_window;
 GtkWidget *dialog_IDinput;
 
+//extern GtkWidget *dialog_record;
+
+
 /* shared memory */
 long *shm_sec;
 long *shm_usec;
@@ -73,12 +77,12 @@ SendAdv ()
     exit (1);
   }
   shm_sec = (long *)shmat(shm_id_s, 0, 0);
-  if (shm_sec == -1) {
+  if ((int)shm_sec == -1) {
     fprintf(stderr, "error : shm_sec\n");
     exit (1);
   }
   shm_usec = (long *)shmat(shm_id_u, 0, 0);
-  if (shm_usec == -1) {
+  if ((int)shm_usec == -1) {
     fprintf(stderr, "error : shm_usec\n");
     exit (1);
   }
@@ -213,8 +217,10 @@ AdvInit ()
 int
 ConnectMyself ()
 {
-  char tmp[64] = "multivnc_client -d 1 --nonotice -u ";
-  strcat (tmp, server_CB.server_ID);
+  char tmp[256] = "multivnc_client -d 1 --nonotice -u ";
+  strcat (tmp, (char *)server_CB.server_ID);
+  strcat (tmp, " -s ");
+  strcat (tmp, (char *)server_CB.server_ID);
   strcat (tmp, " &");
   printf("tmp=%s\n",tmp);
 
@@ -296,7 +302,7 @@ ConnectVNC (struct sockaddr_in client_addr, int vncdsp, int cnum)
 	server_CB.client_grp[cnum]->draw_page = page = tab_num;
 	server_CB.client_grp[cnum]->draw_area = da_num = 0;
 	makeTable(page, 2, 2);
-	ChangeLabel(page, 0, server_CB.client_grp[cnum]->client_ID);
+	ChangeLabel(page, 0, (char *)server_CB.client_grp[cnum]->client_ID);
 	tab_num++;
       }
     }
@@ -500,7 +506,7 @@ controlSocketCallback (gpointer clientData, gint fd, GdkInputCondition condition
       break;
 
     case rfbClientExist:
-      server_CB.client_grp[clientNum]->state &= 0xbf | (buf[1] & 0x40);
+      server_CB.client_grp[clientNum]->state &= ~LOCKSTATE | (buf[1] & LOCKSTATE);
 
       struct timeval tmptime;
       gettimeofday(&timenow, 0);
@@ -508,12 +514,83 @@ controlSocketCallback (gpointer clientData, gint fd, GdkInputCondition condition
       if (tmptime.tv_usec < 0) tmptime.tv_usec += ((timenow.tv_sec - (*shm_sec)) * 1000000);
       server_CB.client_grp[clientNum]->r_time = tmptime.tv_usec;
 
-      if (buf[1] & 0x01) SETRESEND (clientNum, 1);
+      if (buf[1] & RESENDSTATE) SETRESEND (clientNum, 1);
+      if (!(buf[1] & RECSTATE)) SETREC (clientNum, 0);
       break;
 
+#if 0
+    case  rfbFileAdv:
+      {
+	size_t fsize, received = 0;
+	char fname[STRLENGTH +1];
+	char data[SWF_DATA +1];
+	FILE *fp;
+
+	if (read (socket, &fsize, sizeof(fsize)) <= 0) {
+	  fprintf(stderr, "client:%d fsize", clientNum);
+	  perror("read");
+	  return;
+	}
+	if ((received = recv (socket, fname, STRLENGTH, 0)) < STRLENGTH) {
+	  fprintf(stderr, "client:%dB file name", clientNum);
+	  perror("recv");
+	  return;
+	}
+	if ((received = recv (socket, data, SWF_DATA, 0)) < SWF_DATA) {
+	  fprintf(stderr, "client:%dB file data %dB get", clientNum, received);
+	  perror("recv");
+	  return;
+	}
+
+//	fprintf(stderr, "recv(cnum:%d flag:%d fsize:%d(%d) name:%s(%d) data:%d\n", clientNum, buf[1], fsize, sizeof(fsize), fname, sizeof(fname), sizeof(data));
+	switch (buf[1]) {
+	// START or CONTINUE
+	  case START:
+	      fprintf(stderr, "client:%d %s revcieve start.\n", clientNum, fname);
+	  case CONTINUE:
+	    if ((server_CB.client_grp[clientNum]->fp = fopen(fname, "a+")) == NULL) {
+	      fprintf(stderr, "client:%d ", clientNum);
+	      perror("fopen");
+	    }
+	    if(fwrite(data, fsize, 1, server_CB.client_grp[clientNum]->fp) <= 0) {
+//	    if(fwrite(data, received, 1, server_CB.client_grp[clientNum]->fp) <= 0) {
+	      fprintf(stderr, "client:%d ", clientNum);
+	      perror("fwrite");
+	    }
+	    fclose(server_CB.client_grp[clientNum]->fp);
+	    break;
+
+          case STOP:
+	    {
+	      fprintf(stderr, "client:%d %s(%d) revcieve done.\n", clientNum, fname, fsize);
+	      SETREC(clientNum, STOP);
+	      // bottun clear
+	      if (dialog_record != NULL)
+	        on_dialog_rec_stop_button_clicked(NULL, NULL);
+	      /*
+	      int pCnt = server_CB.client_grp[clientNum]->draw_page;
+	      int dCnt = server_CB.client_grp[clientNum]->draw_area;
+              gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dialog_send_checkbutton2[pCnt][dCnt]), GETREC(clientNum));
+	      */
+	      size_t size;
+	      size = get_file_size(fname);
+	      if (size != fsize) {
+	        fprintf(stderr, "%s transfer error.(%d/%d)\n", fname, size, fsize);
+	      }
+	      else {
+	        fprintf(stderr, "%s transfer success.(%d/%d)\n", fname, size, fsize);
+	      }
+	    }
+	    break;
+	}
+      }
+      break;
+
+#endif
+
     default:
       perror("Unknown Message Type");
-      printf(" %d\n",buf[0]);
+      printf("client:%d, type %d\n", clientNum, buf[0]);
       break;
   } 
   server_CB.client_grp[clientNum]->g_ctldsp = gdk_input_add_priority (server_CB.client_grp[clientNum]->ctldsp,
@@ -605,8 +682,8 @@ AuthDB (char *client_ID)
   printf("host=%s, port=%d, name=%s, pass=%s, database=%s\n", \
       server_CB.DB_Host,server_CB.DB_Port,server_CB.DB_Name,server_CB.DB_Pass,server_CB.DB_Database);
   if ((myData = mysql_init((MYSQL *)0)) && \
-      mysql_real_connect (myData, server_CB.DB_Host, server_CB.DB_Name, \
-      server_CB.DB_Pass, server_CB.DB_Database, server_CB.DB_Port, NULL, 0)) {
+      mysql_real_connect (myData, (char *)server_CB.DB_Host, (char *)server_CB.DB_Name, \
+      (char *)server_CB.DB_Pass, (char *)server_CB.DB_Database, server_CB.DB_Port, NULL, 0)) {
     num = mysql_query (myData, Tmpquery);
     if (num != 0) printf("Can't connect SQL-server\n");
     res = mysql_store_result(myData);
@@ -632,14 +709,14 @@ chStateDB(int clientNum, int flag)
 
   if (flag) strcpy(Tmpquery, "update student set joinnow = 't' where joinnow = 'f' and id = '");
   else strcpy(Tmpquery, "update student set joinnow = 'f' where joinnow = 't' and id = '");
-  strcat(Tmpquery, server_CB.client_grp[clientNum]->client_ID);
+  strcat(Tmpquery, (char *)server_CB.client_grp[clientNum]->client_ID);
   strcat(Tmpquery, "'");
 
   printf("%s\n",Tmpquery);
   
   if ((myData = mysql_init((MYSQL *)0)) && \
-      mysql_real_connect (myData, server_CB.DB_Host, server_CB.DB_Name, \
-      server_CB.DB_Pass, server_CB.DB_Database, server_CB.DB_Port, NULL, 0)) {
+      mysql_real_connect (myData, (char *)server_CB.DB_Host, (char *)server_CB.DB_Name, \
+      (char *)server_CB.DB_Pass, (char *)server_CB.DB_Database, server_CB.DB_Port, NULL, 0)) {
     num = mysql_query (myData, Tmpquery);
     if (num == 0) {
       /* success */
@@ -679,7 +756,7 @@ DiscoverDrawingarea(int client)
     area = draw_info[pCnt].row * draw_info[pCnt].column;
     for (dCnt = 0; dCnt < area; dCnt++) {
       if (draw_info[pCnt].client_num[dCnt] == EMPTY) {
-	ChangeLabel(pCnt, dCnt, server_CB.client_grp[client]->client_ID);
+	ChangeLabel(pCnt, dCnt, (char *)server_CB.client_grp[client]->client_ID);
 	server_CB.client_grp[client]->draw_page = page = pCnt;
 	server_CB.client_grp[client]->draw_area = da_num = dCnt;
 	return True;
@@ -720,13 +797,13 @@ CreateMultiSockToSend (unsigned int ip, int port)
   stMreq.imr_multiaddr.s_addr = adv.sin_addr.s_addr;
   stMreq.imr_interface.s_addr = INADDR_ANY;
 
-  if (setsockopt(sockadv, IPPROTO_IP, IP_ADD_MEMBERSHIP, (char*) &stMreq, sizeof(stMreq)) < 0) {
+  if (setsockopt(sockadv, IPPROTO_IP, IP_ADD_MEMBERSHIP, (char *) &stMreq, sizeof(stMreq)) < 0) {
     fprintf(stderr, "%s : CreateMultiSockToSend : ip_add_membership\n", programName);
     close(sockadv);
     exit (1);
   }
 
-  if (setsockopt(sockadv, IPPROTO_IP, IP_MULTICAST_TTL, (char*) &TTLadv, sizeof(TTLadv)) < 0) {
+  if (setsockopt(sockadv, IPPROTO_IP, IP_MULTICAST_TTL, (char *) &TTLadv, sizeof(TTLadv)) < 0) {
     fprintf(stderr, "%s : CreateMultiSockToSend : ip_multicast_ttl\n", programName);
     close(sockadv);
     exit (1);
@@ -761,6 +838,7 @@ SendCastMsg (char flag, int clientNum, char fullscreen)
   castadv.casting_ip = server_CB.casting_ip[server_CB.client_grp[clientNum]->recv_grp];
   castadv.casting_port = server_CB.casting_port;
   castadv.fullscreen = fullscreen;
+  memcpy(&castadv.pixelFormat, &myFormat, sizeof(rfbPixelFormat));
 
   for (i = 0; i < MAXCLIENT; i++) {
     if (server_CB.client_grp[i]->cast_grp == server_CB.client_grp[clientNum]->recv_grp) {
@@ -878,3 +956,5 @@ sem_unlock ()
   semaphore = 1;
 #endif
 }
+
+
